import { jsx as _jsx, jsxs as _jsxs } from "react/jsx-runtime";
import { useCallback, useMemo, useState } from 'react';
import { useNavigate } from 'react-router-dom';
import { observer } from 'mobx-react-lite';
import { Grid, Typography } from '@mui/material';
import { AdminTable, Header } from '../../components';
import AddEditForm, { FormTypeEnum } from '../../components/form/AdminForm';
import { FieldEnum } from '../../constants/common-constants';
import { ToastPreset } from '../../constants/toast-constants';
import { useVsAdmin } from '../../context';
import { useTableData } from '../../hooks/useData';
import { useStore } from '../../models/root-store/root-store';
import { convertSnakeCaseToTitleCase } from '../../utils';
import './details-styles.scss';
const Details = (props) => {
    const { data, title, id, details } = props;
    const { columns, schema, defaultValues, apiEndPoint, key, handleDetailsSubmit, actions } = details;
    const store = useStore();
    const navigation = useNavigate();
    const { getToken } = useVsAdmin();
    const { viewStore } = store;
    const { toastStore } = viewStore;
    const { popToast } = toastStore;
    const { updateItem, isPending } = useTableData({
        getToken,
        key: key,
        apiEndPoint: apiEndPoint,
    });
    const [isLoading, setIsLoading] = useState(false);
    const addEditColumns = columns?.filter(item => !!item.fieldProps);
    let prefillData = (data || defaultValues);
    const [open, setOpen] = useState(false);
    /**
     * Get the header name for the given key
     */
    const getHeaderName = useCallback((key) => columns.find(column => column.field === key)?.headerName || convertSnakeCaseToTitleCase(key), [columns]);
    /**
     * Get the columns for the given key
     */
    const getColumns = useCallback((key) => {
        const foundColumn = columns.find(column => column.field === key);
        return foundColumn?.fieldProps?.fieldType === FieldEnum.REPEATER
            ? foundColumn?.fieldProps?.columns
            : [];
    }, [columns]);
    /**
     * Render the table for the array of objects
     * @param array - array of objects
     * @param key - key of the array
     * @returns JSX.Element
     */
    const renderTableForArrayOfObjects = (array, key) => {
        const columns = getColumns(key);
        const updatedArray = array?.map((obj, rowIndex) => {
            return {
                ...obj,
                id: `${key}-${rowIndex}`,
            };
        });
        const tableProps = {
            rows: updatedArray,
            columns: columns,
        };
        return _jsx(AdminTable, { ...tableProps });
    };
    /**
     * Format the display value based on the type
     * @param key - key of the object
     * @param value - value of the object
     * @param arrayOfObjectsEntries - array of objects
     * @returns string | JSX.Element
     * */
    const formatDisplayValue = (key, value, arrayOfObjectsEntries) => {
        switch (typeof value) {
            case 'boolean':
                return value ? 'True' : 'False';
            case 'object':
                if (value === null)
                    return '-';
                if (Array.isArray(value)) {
                    if (value.every(item => typeof item === 'string')) {
                        return value.length ? value.join(', ') : '-';
                    }
                    else if (value.every(item => typeof item === 'object')) {
                        arrayOfObjectsEntries.push([key, value]);
                        return '-';
                    }
                }
                return Object.keys(value).length ? JSON.stringify(value) : '-';
            default:
                return value || '-';
        }
    };
    /**
     * Memoized values
     * */
    const { gridItems, arrayOfObjectsEntries } = useMemo(() => {
        const arrayOfObjectsEntries = [];
        const gridItems = Object.entries(data).map(([key, value]) => {
            const displayValue = formatDisplayValue(key, value, arrayOfObjectsEntries);
            return (_jsxs(Grid, { item: true, xs: 12, lg: 3, md: 6, sm: 8, className: "detailsPage__dataContainer", children: [_jsx(Typography, { className: "detailsPage__dataKey", children: getHeaderName(key) }), _jsx(Typography, { className: "detailsPage__dataValue", children: displayValue })] }, key));
        });
        return { gridItems, arrayOfObjectsEntries };
    }, [data, getHeaderName]);
    const handleFormClose = () => {
        setOpen(false);
    };
    const afterUpdateSuccess = () => {
        setOpen(false);
        navigation(-1);
        popToast({
            type: ToastPreset.Success,
            message: 'Record updated successfully!',
        });
    };
    const onError = () => {
        popToast({
            type: ToastPreset.Error,
            message: 'Something went wrong!',
        });
    };
    const handleSubmit = () => async (formData) => {
        try {
            setIsLoading(true);
            const updatedFoRmData = handleDetailsSubmit ? await handleDetailsSubmit(formData) : formData;
            updateItem(id, updatedFoRmData, afterUpdateSuccess, onError);
        }
        catch (error) {
            onError();
        }
        finally {
            setIsLoading(false);
        }
    };
    const handleEdit = () => {
        setOpen(true);
    };
    return (_jsxs("div", { className: "detailsPage__container", children: [!!actions.edit && (_jsx(Header, { isPrimaryButtonEnabled: true, primaryButtonTitle: "Edit", onPrimaryButtonPress: handleEdit, containerClassName: "detailsPage__header" })), _jsx(Grid, { container: true, rowSpacing: 3, className: "detailsPage__wrapper", children: gridItems }), arrayOfObjectsEntries?.map(([key, value]) => (_jsxs("div", { className: "detailsPage__arrayOfObjectsContainer", children: [_jsx(Typography, { className: "detailsPage__arrayOfObjectsTitle", children: getHeaderName(key) }), renderTableForArrayOfObjects(value, key)] }, key))), _jsx(AddEditForm, { onClose: handleFormClose, open: open, fields: addEditColumns, schema: schema, defaultValues: prefillData, type: FormTypeEnum.Edit, onSubmit: handleSubmit, title: title, secondaryButtonTitle: 'Cancel', onSecondaryButtonClick: handleFormClose, isLoading: isPending || isLoading })] }));
};
export default observer(Details);
